#include <capdev_pcap.h>
#include <s_util.h>

proc_print_out print_out;
proc_print_raw print_raw;
proc_add_captype add_captype;

void *lp_init(char *interface, char *filter)
{
	p_cpcap_priv priv;
	char buf[4096];
	char *err=(char *)&buf;
	
	if ( !(priv=malloc(sizeof(cpcap_priv_t))) ) return NULL;
	memset(priv, 0, sizeof(cpcap_priv_t));

	priv->pcap_desc=pcap_open_live(interface, CAP_SNAPLEN, 0, 1, err);

	if (!priv->pcap_desc) {
		print_out(err);
		print_out(CRLF);
		free(priv);
		return NULL;
	}
	
	priv->link = pcap_datalink(priv->pcap_desc);
	
	return priv;
}

int lp_end(void *priv)
{
	p_cpcap_priv p=(p_cpcap_priv)priv;

	if ( p->pcap_desc ) 
		pcap_close(p->pcap_desc);

	free(p);
	
	return 1;
}

void lp_callback(u_char *user, struct pcap_pkthdr *header, u_char *data)
{
	p_cpcap_priv p;
	uint real_len;

	if ( !(p=(p_cpcap_priv)user) ) return;

	time (&p->pkt.time);
	p->pkt.ltime=localtime(&p->pkt.time);

	real_len=p->pkt.caplen=header->caplen;
	p->pkt.len=header->len;

	p->pkt.layer[LAYER_LINK].id = p->link;
	
	if ( header->caplen > CAP_SNAPLEN ) {
		real_len = CAP_SNAPLEN;
	}
	
	memcpy( &p->pkt.data, data, real_len);
}

struct packet *lp_poll(void *priv)
{
	p_cpcap_priv p=NULL;

	if ( !(p=(p_cpcap_priv)priv) ) return NULL;
	
	if ( !pcap_dispatch(p->pcap_desc, 0, lp_callback, priv) ) {
		return NULL;
	}else{
		return &p->pkt;
	}
}

uint _PLUGIN_INIT (p_plugin_in in, p_plugin_out out)
{
	PLUGIN_CHECK_PARAMS(in);
	
	print_out=in->print_out;
	print_raw=in->print_raw;
	add_captype=in->add_captype;
	
	out->name="capture/libpcap";
	out->desc="Capture packets using libpcap";
	out->ver_major=1;
	out->ver_minor=0;

	if ( add_captype("libpcap_live", lp_init, lp_end, lp_poll) ) {
		return PLUGIN_ERR_OK;
	}else{
		return PLUGIN_ERR_FAIL;
	}
}

/* This is where we would cleanup our shit */
uint _PLUGIN_UNLOAD (void)
{
	return PLUGIN_ERR_OK;
}
