#ifndef __GLOBAL_HEADER_INCLUDED__
#define __GLOBAL_HEADER_INCLUDED__

/* Autoconf header */
#ifdef HAVE_CONFIG_H
#include <config.h>
#else
#error "EEK, Where is config.h?!"
#endif

#define GNU_STRING		"Released under the terms of the GNU GPL" CRLF
#define COPYRIGHT_STRING	"Copyright (c) 2001 Gianni Tedesco" CRLF GNU_STRING
#define VERSION_STRING		"Firestorm v" VERSION CRLF COPYRIGHT_STRING

/* ANSI Headers */
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>

/* POSIX Headers */
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include <signal.h>
#include <netdb.h>

/* POSIX Threads */
#define _REENTRANT
#define _POSIX_THREAD_SEMANTICS
#include <pthread.h>

#if (HAVE_DLFCN_H == 1)
#include <dlfcn.h>
#else
#error "You need libdl, try freshmeat.net"
#endif

#include <pcap.h>

#define uchar unsigned char

#define BUFFER_SIZE	4096
#define CRLF		"\n"

/******************************************
 * Exit codes
 *******************************************/
#define EXIT_OK		0
#define EXIT_ERR	-1
#define EXIT_DBG	1

/******************************************
 * Bitmask macros
 *******************************************/
#define MASK(x,y) (x & y)
#define MASK_SET(x,y) x |= y
#define MASK_UNSET(x,y) x &= ~y
#define GEN_IPMASK(x) ~(0xFFFFFFFF << x)


/******************************************
 * Generic Mutex encapsulations
 *******************************************/
typedef struct mlock {
	char *debug;
	int enabled;
	pthread_mutex_t mutex;
}mlock_t, *pmlock;

typedef struct atomic_counter {
	mlock_t		lock;
	uint		counter;
}atomic_counter_t, *p_atomic_counter;

#define CRITICAL_START(x) if (x.enabled){pthread_mutex_lock(&x.mutex);}
#define CRITICAL_STOP(x) if (x.enabled){pthread_mutex_unlock( &x.mutex );}

#define MUTEX_ENABLE(x) x.enabled=1;
#define MUTEX_DISABLE(x) x.enabled=0

#define MUTEX_DESTROY(x) pthread_mutex_destroy(&x.mutex); MUTEX_DISABLE(x);
#define MUTEX_CREATE(x,y) pthread_mutex_init(&x.mutex, NULL); x.debug=y; MUTEX_ENABLE(x);

#define MUTEX_PENABLE(x) x->enabled=1;
#define MUTEX_PDISABLE(x) x->enabled=0;

#define MUTEX_PDESTROY(x) pthread_mutex_destroy(&x->mutex); MUTEX_PDISABLE(x);
	
#endif //__GLOBAL_HEADER_INCLUDED__
