#include <match_tcp.h>
#include <s_util.h>

proc_print_out		print_out;
proc_print_raw		print_raw;
proc_matcher_add	add_matcher;

#define CHECK_TCP(x) \
	(( x->layer[LAYER_NET].id == 0x0800 ) ? 1 : 0) && \
	(( x->layer[LAYER_TRANS].id == IPPROTO_TCP ) ? 1 : 0)

void *flags_validate(char *token, p_kv opts)
{
	p_tcpflag_priv priv;
	char *tmp;
	
	if ( !(priv=malloc(sizeof(tcpflag_priv_t))) ) return NULL;
	priv->mode=MODE_NORMAL;
	
	for (tmp=token; *tmp != '\0'; tmp++ ) {
		switch(*tmp) {
			case 'f':
			case 'F':
				priv->tcp_flags != R_FIN;
				break;
				
			case 's':
			case 'S':
				priv->tcp_flags |= R_SYN;
				break;
				
			case 'r':
			case 'R':
				priv->tcp_flags |= R_RST;
				break;

			case 'p':
			case 'P':
				priv->tcp_flags != R_PSH;
				break;

			case 'a':
			case 'A':
				priv->tcp_flags |= R_ACK;
				break;

			case 'u':
			case 'U':
				priv->tcp_flags |= R_URG;
				break;

			case '1':
			case 'E':
			case 'e':
				priv->tcp_flags |= R_RES1;
				break;

			case '2':
			case 'C':
			case 'c':
				priv->tcp_flags |= R_RES2;
				break;

			case '!':
				priv->mode = MODE_NOT;
				break;

			case '*':
				priv->mode = MODE_ANY;
				break;

			case '+':
				priv->mode = MODE_ALL;
				break;

			case ' ':
			case '\t':
				break;
			default:
				free(priv);
				return NULL;
		}
	}
	
	return priv;
}

int ack_match(void *priv, p_packet p, int negate)
{
	uint ack;
	p_range_priv x=(p_range_priv)priv;
	
	if ( !CHECK_TCP(p) ) return 0;

	ack=ntohl(p->layer[LAYER_TRANS].h.tcp->th_ack);
	
	if ( (ack >= x->min) && (ack <= x->max) ) {
		MATCH_RET_POSITIVE;
	}else{
		MATCH_RET_NEGATIVE;
	}
}

int win_match(void *priv, p_packet p, int negate)
{
	ushort win;
	p_range_priv x=(p_range_priv)priv;
	
	if ( !CHECK_TCP(p) ) return 0;

	win=ntohs(p->layer[LAYER_TRANS].h.tcp->th_win);
	
	if ( (win >= x->min) && (win <= x->max) ) {
		MATCH_RET_POSITIVE;
	}else{
		MATCH_RET_NEGATIVE;
	}
}

int urp_match(void *priv, p_packet p, int negate)
{
	ushort urp;
	p_range_priv x=(p_range_priv)priv;
	
	if ( !CHECK_TCP(p) ) return 0;

	urp=ntohs(p->layer[LAYER_TRANS].h.tcp->th_urp);
	
	if ( (urp >= x->min) && (urp <= x->max) ) {
		MATCH_RET_POSITIVE;
	}else{
		MATCH_RET_NEGATIVE;
	}
}


int seq_match(void *priv, p_packet p, int negate)
{
	uint seq;
	p_range_priv x=(p_range_priv)priv;
	
	if ( !CHECK_TCP(p) ) return 0;

	seq=ntohl(p->layer[LAYER_TRANS].h.tcp->th_seq);
	
	if ( (seq >= x->min) && (seq <= x->max) ) {
		MATCH_RET_POSITIVE;
	}else{
		MATCH_RET_NEGATIVE;
	}
}


int flags_match(void *priv, p_packet p, int negate)
{
	p_tcpflag_priv x=priv;
	unsigned char flags;
	
	if ( !CHECK_TCP(p) ) return 0;

	flags=p->layer[LAYER_TRANS].h.tcp->th_flags;
	
	switch ( x->mode ) {
		case MODE_NORMAL:
			if ( flags == x->tcp_flags ) {
				MATCH_RET_POSITIVE;
			}else{
				MATCH_RET_NEGATIVE;
			}
		case MODE_ALL:
			if ( (flags & x->tcp_flags) == x->tcp_flags ) {
				MATCH_RET_POSITIVE;
			}else{
				MATCH_RET_NEGATIVE;
			}
		case MODE_NOT:
			if ( (flags & x->tcp_flags) == 0) {
				MATCH_RET_POSITIVE;
			}else{
				MATCH_RET_NEGATIVE;
			}
		case MODE_ANY:
			if ( (flags & x->tcp_flags) != 0) {
				MATCH_RET_POSITIVE;
			}else{
				MATCH_RET_NEGATIVE;
			}
	}
	
	print_out(PRINT_WARNING "Wrong TCP mode, should never happen" CRLF);
	return 0;
}
	
uint _PLUGIN_INIT (p_plugin_in in, p_plugin_out out)
{
	PLUGIN_CHECK_PARAMS(in);

	print_out=in->print_out;
	print_raw=in->print_raw;
	add_matcher=in->add_matcher;

	out->name="match/tcp";
	out->desc="Some generic TCP based matchers";
	out->ver_major=1;
	out->ver_minor=0;
	
	add_matcher("FLAGS",
		flags_validate,
		in->templates.t_cleanup_ptr,
		flags_match);
	
	add_matcher("SEQ",
		in->templates.t_validate_range,
		in->templates.t_cleanup_ptr,
		seq_match);

	add_matcher("ACK",
		in->templates.t_validate_range,
		in->templates.t_cleanup_ptr,
		ack_match);
	
	add_matcher("WIN",
		in->templates.t_validate_range,
		in->templates.t_cleanup_ptr,
		win_match);
	
	add_matcher("URP",
		in->templates.t_validate_range,
		in->templates.t_cleanup_ptr,
		urp_match);

	
	
	return PLUGIN_ERR_OK;
}

/* This is where we would cleanup our shit */
uint _PLUGIN_UNLOAD (void)
{
	return PLUGIN_ERR_OK;
}
